#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/time.h>
#include <time.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/extensions/XShm.h>
#include <X11/extensions/Xrender.h>

typedef struct
{
   const char *test;
   int (*func) (void);
} Test;

#include "x.c"

#include "t_fill.c"
#include "t_copy.c"
#include "t_blend.c"

const Test tests[] = 
{
   {"fill-red", fill_red},
   {"fill-green", fill_green},
   {"fill-blue", fill_blue},
   {"fill-yellow", fill_yellow},
   {"fill-cyan", fill_cyan},
   {"fill-violet", fill_violet},
   {"fill-white", fill_white},
   {"fill-black", fill_black},
   {"fill-pattern", fill_pattern},
   {"copy-small-default-depth", copy_small_default_depth},
   {"copy-medium-default-depth", copy_medium_default_depth},
   {"copy-large-default-depth", copy_large_default_depth},
   {"copy-huge-default-depth", copy_huge_default_depth},
   {"copy-huge-default-depth-scale-down-16", copy_huge_default_depth_scale_down_16},
   {"copy-huge-default-depth-scale-down-8", copy_huge_default_depth_scale_down_8},
   {"copy-huge-default-depth-scale-down-4", copy_huge_default_depth_scale_down_4},
   {"copy-huge-default-depth-scale-down-2", copy_huge_default_depth_scale_down_2},
   {"copy-huge-default-depth-noscale", copy_huge_default_depth_noscale},
   {"copy-huge-default-depth-scale-up-2", copy_huge_default_depth_scale_up_2},
   {"copy-huge-default-depth-scale-up-4", copy_huge_default_depth_scale_up_4},
   {"copy-huge-default-depth-scale-up-8", copy_huge_default_depth_scale_up_8},
   {"copy-huge-default-depth-scale-up-16", copy_huge_default_depth_scale_up_16},
   {"copy-huge-default-depth-scale-down-mini-16", copy_huge_default_depth_scale_down_mini_16},
   {"copy-huge-default-depth-scale-down-mini-8", copy_huge_default_depth_scale_down_mini_8},
   {"copy-huge-default-depth-scale-down-mini-4", copy_huge_default_depth_scale_down_mini_4},
   {"copy-huge-default-depth-scale-down-mini-2", copy_huge_default_depth_scale_down_mini_2},
   {"copy-huge-default-depth-noscale-mini", copy_huge_default_depth_noscale_mini},
   {"copy-huge-default-depth-scale-up-mini-2", copy_huge_default_depth_scale_up_mini_2},
   {"copy-huge-default-depth-scale-up-mini-4", copy_huge_default_depth_scale_up_mini_4},
   {"copy-huge-default-depth-scale-up-mini-8", copy_huge_default_depth_scale_up_mini_8},
   {"copy-huge-default-depth-scale-up-mini-16", copy_huge_default_depth_scale_up_mini_16},
   {"blend-small-default-depth", blend_small_default_depth},
   {"blend-medium-default-depth", blend_medium_default_depth},
   {"blend-large-default-depth", blend_large_default_depth},
   {"blend-huge-default-depth", blend_huge_default_depth},
};



int main(int argc, char **argv)
{
   int i, j;
   int did_tests = 0;
   
   setup();
   
   // do specifc listed tests
   for (i = 1; i < argc; i++)
     {
	if (!strcmp(argv[i], "-h"))
	  {
	     printf
	       ("Usage: glamo-xr-test [OPTIONS] [TEST-NAME [TEST-NAME ...]]\n"
		"  OPTIONS can be one of:\n"
		"  -h          This help\n"
		"  -l X        All looped speed tests only do X loops\n"
		"\n"
		"  TEST-NAME can be 1 or more of:\n"
		"\n"
		"  If no test names are provided, all tests will be run\n"
		);
	     for (j = 0; j < (sizeof(tests) / sizeof(Test)); j++)
	       printf("    %s\n", tests[j].test);
	     return 0;
	  }
	else if ((!strcmp(argv[i], "-l")) && (i < (argc - 1)))
	  {
	     i++;
	     loop_count = atoi(argv[i]);
	  }
	else
	  {
	     for (j = 0; j < (sizeof(tests) / sizeof(Test)); j++)
	       {
		  if (!strcmp(argv[i], tests[j].test))
		    {
		       if (!did_tests)
			 {
			    printf("TEST NAME, RESULT TYPE, RESULT, NOTES\n");
			    did_tests = 1;
			 }
		       printf("%s, ", tests[j].test);
		       if (!tests[j].func())
			 {
			    printf("\n");
			    return -1; // failed
			 }
		       printf("\n");
		    }
	       }
	  }
     }
   // no args - do all tests
   if (!did_tests)
     {
	printf("TEST NAME, RESULT TYPE, RESULT, NOTES\n");
	for (j = 0; j < (sizeof(tests) / sizeof(Test)); j++)
	  {
	     printf("%s, ", tests[j].test);
	     if (!tests[j].func())
	       {
		  printf("\n");
		  return -1; // failed
	       }
	     printf("\n");
	  }
	printf("--- PRESS CTRL-C TO QUIT ---\n"); pause();
	return 0;
     }
   printf("--- PRESS CTRL-C TO QUIT ---\n"); pause();
   return 0;
}
