double get_time(void)
{
   struct timeval      timev;
   gettimeofday(&timev, NULL);
   return (double)timev.tv_sec + (((double)timev.tv_usec) / 1000000);
}

#define WINW 480
#define WINH 640

#define SYNC XSync(disp, False)
#define LOOP1 { double tstart = get_time(); int loops; \
                for (loops = 0; loops < loop_count; loops++) {
#define LOOP2 } SYNC; printf("TIME, %3.3f, ", get_time() - tstart); }
#define CHECK printf("VISUAL, CHECK");
#define NOTE(xxx) printf(", %s", xxx);
#define SHOW XRenderComposite(disp, PictOpSrc, pixmap_pic, None, \
			      win_pic, 0, 0, 0, 0, 0, 0, WINW, WINH); \
             XSync(disp, False)

int loop_count = 4096;
Display *disp = NULL;
Window win = 0;
GC gc = 0;
Pixmap pixmap = 0;

XRenderPictFormat *fmt32 = NULL;
XRenderPictFormat *fmt24 = NULL;
XRenderPictFormat *fmt8 = NULL;
XRenderPictFormat *fmt4 = NULL;
XRenderPictFormat *fmt1 = NULL;
XRenderPictFormat *fmtdef = NULL;

Picture win_pic = 0;
Picture pixmap_pic = 0;

Pixmap tmp_pixmap = 0;
Picture tmp_pic = 0;

void setup(void)
{
   XSetWindowAttributes attr;
   XClassHint chint;
   XSizeHints szhints;
   XRenderPictureAttributes att;
   
   disp = XOpenDisplay(NULL);
   
   if (DefaultDepth(disp, DefaultScreen(disp)) != 16)
     {
	printf("-- WARNING: Default depth not 16 --\n");
     }
   
   attr.backing_store = NotUseful; attr.border_pixel = 0;
   attr.background_pixmap = None; attr.event_mask = 0;
   attr.override_redirect = 1; attr.bit_gravity = ForgetGravity;
   win = XCreateWindow(disp, DefaultRootWindow(disp),
		       0, 0, WINW, WINH, 0, 0, InputOutput, 
		       DefaultVisual(disp, DefaultScreen(disp)),
		       CWBackingStore | CWBackPixmap | CWBorderPixel | 
		       CWBitGravity | CWEventMask | CWOverrideRedirect,
		       &attr);
   XMapWindow(disp, win);
   
   gc = XCreateGC(disp, win, 0, NULL);
   
   pixmap = XCreatePixmap(disp, win, WINW, WINH, 
			  DefaultDepth(disp, DefaultScreen(disp)));

   fmt32 = XRenderFindStandardFormat(disp, PictStandardARGB32);
   fmt24 = XRenderFindStandardFormat(disp, PictStandardRGB24);
   fmt8 = XRenderFindStandardFormat(disp, PictStandardA8);
   fmt4 = XRenderFindStandardFormat(disp, PictStandardA4);
   fmt1 = XRenderFindStandardFormat(disp, PictStandardA1);
   fmtdef = XRenderFindVisualFormat(disp, 
				    DefaultVisual(disp, DefaultScreen(disp)));
   
   att.dither = 1; att.component_alpha = 0; att.repeat = 0;
   win_pic = XRenderCreatePicture(disp, win, fmtdef, 
				  CPRepeat | CPDither | CPComponentAlpha,
				  &att);
   pixmap_pic = XRenderCreatePicture(disp, pixmap, fmtdef, 
				     CPRepeat | CPDither | CPComponentAlpha,
				     &att);
   
   XSync(disp, False);
}

void tmp_new(int w, int h, XRenderPictFormat *fmt)
{
   XRenderPictureAttributes att;
   XTransform xf;      
   int i, j;
   
   tmp_pixmap = XCreatePixmap(disp, win, w, h, fmt->depth);
   att.dither = 1; att.component_alpha = 0; att.repeat = 0;
   tmp_pic = XRenderCreatePicture(disp, tmp_pixmap, fmt,
				  CPRepeat | CPDither | CPComponentAlpha,
				  &att);
   XRenderSetPictureFilter(disp, tmp_pic, FilterNearest, NULL, 0);
   for (i = 0; i < 3; i++)
     {
	for (j = 0; j < 3; j++)
	  {
	     xf.matrix[i][j] = XDoubleToFixed((i == j) ? 1 : 0);
	  }
     }
   XRenderSetPictureTransform(disp, tmp_pic, &xf);
}

void tmp_free(void)
{
   XRenderFreePicture(disp, tmp_pic);
   XFreePixmap(disp, tmp_pixmap);
}

void fill_argb(Picture dst, int x, int y, int w, int h, int a, int r, int g, int b)
{
   XRenderColor col;

   col.red   = (r << 8) | r; col.green = (g << 8) | g;
   col.blue  = (b << 8) | b; col.alpha = (a << 8) | a;
   XRenderFillRectangle(disp, PictOpSrc, dst, &col, x, y, w, h);
}

void fill_test_pattern(Picture dst, int w, int h)
{
   int x, y;
   int c;

   // draw a known image/pattern on the destination
   // white bg
   fill_argb(dst, 0, 0, w, h, 255, 255, 255, 255);
   // black -> white vertical gradient
   for (y = 0; y < h; y++)
     {
	if (h > 0) c = (y * 255) / (h - 1);
	else c = 0;
	if (c > 255) c = 255;
	else if (c < 0) c = 0;
	fill_argb(dst, 0, y, w, 1, 255, c, c, c);
     }
   // vertical color list
   fill_argb(dst, 0, 0 * (h / 8), w / 4, h / 8, 255, 255,   0,   0);
   fill_argb(dst, 0, 1 * (h / 8), w / 4, h / 8, 255,   0, 255,   0);
   fill_argb(dst, 0, 2 * (h / 8), w / 4, h / 8, 255,   0,   0, 255);
   fill_argb(dst, 0, 3 * (h / 8), w / 4, h / 8, 255, 255, 255,   0);
   fill_argb(dst, 0, 4 * (h / 8), w / 4, h / 8, 255,   0, 255, 255);
   fill_argb(dst, 0, 5 * (h / 8), w / 4, h / 8, 255, 255,   0, 255);
   fill_argb(dst, 0, 6 * (h / 8), w / 4, h / 8, 255, 255, 255, 255);
   fill_argb(dst, 0, 7 * (h / 8), w / 4, h / 8, 255,   0,   0,   0);

   // middle grey checker box
   fill_argb(dst, 1 * (w / 4), 1 * (h / 4), w / 4, h / 4, 255, 160, 160, 160);
   fill_argb(dst, 2 * (w / 4), 1 * (h / 4), w / 4, h / 4, 255,  96,  96,  96);
   fill_argb(dst, 1 * (w / 4), 2 * (h / 4), w / 4, h / 4, 255,  96,  96,  96);
   fill_argb(dst, 2 * (w / 4), 2 * (h / 4), w / 4, h / 4, 255, 160, 160, 160);

   fill_argb(dst, 1 * (w / 4),       1 * (h / 4),       w / 2,     1, 255, 255, 255, 255);
   fill_argb(dst, 1 * (w / 4),       (3 * (h / 4)) - 1, w / 2,     1, 255, 255, 255, 255);
   fill_argb(dst, 1 * (w / 4),       1 * (h / 4),           1, h / 2, 255, 255, 255, 255);
   fill_argb(dst, (3 * (w / 4)) - 1, 1 * (h / 4),           1, h / 2, 255, 255, 255, 255);
   
   // a border
   fill_argb(dst,     0,     0, 1, h, 255, 255, 128,   0);
   fill_argb(dst, w - 1,     0, 1, h, 255, 255, 128,   0);   
   fill_argb(dst,     0,     0, w, 1, 255,   0, 128, 255);
   fill_argb(dst,     0, h - 1, w, 1, 255,   0, 128, 255);
   
   // a small diagonal set of pixels top-left from white -> yellow -> orange
   fill_argb(dst,     0,     0, 1, 1, 255, 255, 255, 255);
   fill_argb(dst,     1,     1, 1, 1, 255, 255, 255, 100);
   fill_argb(dst,     2,     2, 1, 1, 255, 255, 200,  50);
   fill_argb(dst,     3,     3, 1, 1, 255, 255, 100,  20);
}

void fill_alpha_test_pattern(Picture dst, int w, int h)
{
   int x, y;
   int c;
   
   for (y = 0; y < h; y++)
     {
	if (h > 0) c = (y * 255) / (h - 1);
	else c = 0;
	if (c > 255) c = 255;
	else if (c < 0) c = 0;
	fill_argb(dst, 0, y, w, 1, c, c, c, c);
     }

   // middle grey checker box
   fill_argb(dst, 1 * (w / 4), 1 * (h / 4), w / 4, h / 4, 255, 160, 160, 160);
   fill_argb(dst, 2 * (w / 4), 1 * (h / 4), w / 4, h / 4, 255,  96,  96,  96);
   fill_argb(dst, 1 * (w / 4), 2 * (h / 4), w / 4, h / 4, 255,  96,  96,  96);
   fill_argb(dst, 2 * (w / 4), 2 * (h / 4), w / 4, h / 4, 255, 160, 160, 160);

   fill_argb(dst, 1 * (w / 4),       1 * (h / 4),       w / 2,     1, 255, 255, 255, 255);
   fill_argb(dst, 1 * (w / 4),       (3 * (h / 4)) - 1, w / 2,     1, 255, 255, 255, 255);
   fill_argb(dst, 1 * (w / 4),       1 * (h / 4),           1, h / 2, 255, 255, 255, 255);
   fill_argb(dst, (3 * (w / 4)) - 1, 1 * (h / 4),           1, h / 2, 255, 255, 255, 255);

   for (y = 0; y < h; y++)
     {
	if (h > 0) c = (y * 255) / (h - 1);
	else c = 0;
	if (c > 255) c = 255;
	else if (c < 0) c = 0;
	fill_argb(dst, 3 * (w / 4), y, w / 4, 1, c, 0, 0, 0);
     }
}

void blit(Picture src, Picture dst, int sx, int sy, int sw, int sh, int x, int y, int w, int h)
{
   int i, j;
   int is_scaling;
   int op;
   XTransform xf;
   
   if ((sw <= 0) || (sh <= 0) || (w <= 0) || (h <= 0)) return;
   
   for (i = 0; i < 3; i++)
     {
	for (j = 0; j < 3; j++)
	  {
	     xf.matrix[i][j] = XDoubleToFixed((i == j) ? 1 : 0);
	  }
     }
   xf.matrix[0][0] = XDoubleToFixed((double)(sw) / (double)(w));
   xf.matrix[1][1] = XDoubleToFixed((double)(sh) / (double)(h));
   xf.matrix[2][0] = (0 * sw) / w;
   xf.matrix[2][1] = (0 * sh) / h;
   XRenderSetPictureFilter(disp, src, FilterNearest, NULL, 0);
//   XRenderSetPictureFilter(disp, src, FilterBest, NULL, 0);
   XRenderSetPictureTransform(disp, src, &xf);
   XRenderComposite(disp, PictOpSrc, src, None, dst, 
		    (sx + 1 + w) / sw, (sy + 1 + h) / sw, 
		    (sx + 1 + w) / sw, (sy + 1 + h) / sw, 
		    x, y, w, h);
}

void composite(Picture src, Picture dst, int sx, int sy, int sw, int sh, int x, int y, int w, int h)
{
   int i, j;
   int is_scaling;
   int op;
   XTransform xf;
   
   if ((sw <= 0) || (sh <= 0) || (w <= 0) || (h <= 0)) return;
   
   for (i = 0; i < 3; i++) for (j = 0; j < 3; j++)
     xf.matrix[i][j] = XDoubleToFixed((i == j) ? 1 : 0);
   xf.matrix[0][0] = XDoubleToFixed((double)(sw) / (double)(w));
   xf.matrix[1][1] = XDoubleToFixed((double)(sh) / (double)(h));
   xf.matrix[2][0] = (0 * sw) / w;
   xf.matrix[2][1] = (0 * sh) / h;
   XRenderSetPictureFilter(disp, src, FilterNearest, NULL, 0);
//   XRenderSetPictureFilter(disp, src, FilterBest, NULL, 0);
   XRenderSetPictureTransform(disp, src, &xf);
   XRenderComposite(disp, PictOpOver, src, None, dst, 
		    (sx + 1 + w) / sw, (sy + 1 + h) / sw, 
		    (sx + 1 + w) / sw, (sy + 1 + h) / sw, 
		    x, y, w, h);
}
